<?php

namespace Axepta\Payment;

/**
 * Class Api
 *
 * @package Axepta\Payment
 */
class Api
{

    const PAYMENT = 'payment';
    const PROFILE = 'profile';
    const MERCHANT = 'merchant';
    const SERVICE = 'service';
    const TRANSACTION_TYPE_SALE = 'sale';
    const TRANSACTION_TYPE_REFUND = 'refund';

    const ACTION_REFUND = 'refund';

    /**
     * @var array
     */
    private static $serviceUrls = [
        Util::ENVIRONMENT_PRODUCTION => 'https://api.axepta.pl/v1',
        Util::ENVIRONMENT_SANDBOX    => 'https://api.sandbox.axepta.pl/v1',
    ];

    /**
     * @var string
     */
    private $authorizationToken;

    /**
     * @var string
     */
    private $merchantId;

    /**
     * @var string
     */
    private $serviceId;

    /**
     * @var string
     */
    private $environment;

    /**
     * Api constructor.
     *
     * @param string $authorizationToken
     * @param string $merchantId
     * @param string $serviceId
     * @param string $environment
     */
    public function __construct($authorizationToken, $merchantId, $serviceId, $environment = '')
    {

        $this->authorizationToken = $authorizationToken;
        $this->merchantId = $merchantId;
        $this->serviceId = $serviceId;

        if (!$environment) {
            $environment = Util::ENVIRONMENT_PRODUCTION;
        }

        $this->environment = $environment;
    }

    /**
     * @param array $transaction
     *
     * @return array
     */
    public static function parseStringToArray($transaction)
    {

        $array = [];

        if ($transaction['action']['method'] === Util::METHOD_REQUEST_POST) {
            parse_str($transaction['action']['contentBodyRaw'], $array);
        }

        if ($transaction['action']['method'] === Util::METHOD_REQUEST_GET) {
            $urlParsed = parse_url($transaction['action']['url']);

            if (isset($urlParsed['query']) && $urlParsed['query']) {
                parse_str($urlParsed['query'], $array);
            }
        }

        return $array;
    }

    /**
     * @param int $amount
     *
     * @return string
     */
    public function prepareRefundData(
        $amount
    ) {

        return json_encode([
            'amount'    => $amount,
            'serviceId' => $this->serviceId,
            'type'      => self::TRANSACTION_TYPE_REFUND,
        ]);
    }
    // endregion

    /**
     * @param string $body
     * @param string $quotaUuid
     *
     * @return array
     */
    public function createRefund($body, $quotaUuid)
    {

        return $this->call(
            $this->getActionUrl($quotaUuid, self::ACTION_REFUND),
            Util::METHOD_REQUEST_POST,
            $body
        );
    }

    /**
     * @param string $url
     * @param string $methodRequest
     * @param string $body
     *
     * @return array
     */
    private function call($url, $methodRequest, $body = '')
    {

        $curl = curl_init($url);
        curl_setopt($curl, CURLOPT_CUSTOMREQUEST, $methodRequest);
        curl_setopt($curl, CURLOPT_POSTFIELDS, $body);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($curl, CURLOPT_CONNECTTIMEOUT, 10);
        curl_setopt($curl, CURLOPT_TIMEOUT, 10);
        curl_setopt($curl, CURLOPT_HTTPHEADER, [
            'Content-Type: application/json',
            'Authorization: Bearer ' . $this->authorizationToken,
        ]);

        $resultCurl = json_decode(curl_exec($curl), true);

        $httpCode = curl_getinfo($curl, CURLINFO_HTTP_CODE);

        if (($httpCode !== 200) || !$resultCurl) {

            $array = [
                'success' => false,
                'data'    => [
                    'httpCode' => $httpCode,
                    'error'    => curl_error($curl),
                    'body'     => '',
                ],
            ];

            if (isset($resultCurl['apiErrorResponse']['message']) && $resultCurl['apiErrorResponse']['message']) {
                $array['data']['body'] = $resultCurl['apiErrorResponse']['message'];
            }

            return $array;
        }

        return [
            'success' => true,
            'body'    => $resultCurl,
        ];
    }

    /**
     * @param string $quotaUuid
     * @param string $action
     *
     * @return string
     */
    private function getActionUrl($quotaUuid, $action)
    {

        $baseUrl = $this->getServiceUrl();

        if ($baseUrl) {

            switch ($action) {
                case self::ACTION_REFUND:
                    return $baseUrl
                        . '/'
                        . self::MERCHANT
                        . '/'
                        . $this->merchantId
                        . '/'
                        . self::PAYMENT
                        . '/'
                        . $quotaUuid
                        . '/refund';
                default:
                    return '';
            }
        }

        return '';
    }

    /**
     * @return string
     */
    private function getServiceUrl()
    {

        if (isset(self::$serviceUrls[$this->environment])) {
            return self::$serviceUrls[$this->environment];
        }

        return '';
    }
}
